@@ -1,6 +1,6 @@
-/* Copyright (C) 2001-2008 Artifex Software, Inc.
+/* Copyright (C) 2001-2011 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: dwmain.c 9043 2008-08-28 22:48:19Z giles $ */
+/* $Id$ */
 /* Ghostscript DLL loader for Windows */
 
 #include "windows_.h"
@@ -40,11 +40,15 @@ TW *tw;
 
 static const LPSTR szAppName = "Ghostscript";
 
+#ifdef _WIN64
+const LPSTR szIniName = "gswin64.ini";
+const char *szDllName = "gsdll64.dll";
+#else
 const LPSTR szIniName = "gswin32.ini";
 const char *szDllName = "gsdll32.dll";
+#endif
 const LPSTR szIniSection = "Text";
 
-
 GSDLL gsdll;
 void *instance;
 HWND hwndtext;
@@ -55,12 +59,12 @@ static int poll(void)
 {
     MSG msg;
     while (PeekMessage(&msg, 0, 0, 0, PM_REMOVE)) {
-	TranslateMessage(&msg);
-	DispatchMessage(&msg);
+        TranslateMessage(&msg);
+        DispatchMessage(&msg);
     }
     /* If text window closing then abort Ghostscript */
     if (tw->quitnow)
-	return e_Fatal;
+        return e_Fatal;
     return 0;
 }
 
@@ -95,7 +99,7 @@ static int GSDLLCALL gsdll_poll(void *handle)
 /*********************************************************************/
 
 /* new dll display device */
-/* 
+/*
 #define DISPLAY_DEBUG
  */
 
@@ -140,37 +144,37 @@ static int display_close(void *handle, void *device)
 #endif
     img = image_find(handle, device);
     if (img) {
-	image_delete(img);	/* remove from list but don't free */
-	image_close(img);
+        image_delete(img);	/* remove from list but don't free */
+        image_close(img);
     }
     return 0;
 }
 
 /* Device is about to be resized. */
 /* Resize will only occur if this function returns 0. */
-static int display_presize(void *handle, void *device, int width, int height, 
-	int raster, unsigned int format)
+static int display_presize(void *handle, void *device, int width, int height,
+        int raster, unsigned int format)
 {
 #ifdef DISPLAY_DEBUG
     char buf[256];
     sprintf(buf, "display_presize(0x%x, 0x%x, width=%d height=%d raster=%d\n\
-  format=%d)\n", 
+  format=%d)\n",
        handle, device, width, height, raster, format);
     text_puts(tw, buf);
 #endif
     return 0;
 }
-   
+
 /* Device has been resized. */
 /* New pointer to raster returned in pimage */
-static int display_size(void *handle, void *device, int width, int height, 
-	int raster, unsigned int format, unsigned char *pimage)
+static int display_size(void *handle, void *device, int width, int height,
+        int raster, unsigned int format, unsigned char *pimage)
 {
     IMAGE *img;
 #ifdef DISPLAY_DEBUG
     char buf[256];
     sprintf(buf, "display_size(0x%x, 0x%x, width=%d height=%d raster=%d\n\
-  format=%d image=0x%x)\n", 
+  format=%d image=0x%x)\n",
        handle, device, width, height, raster, format, pimage);
     text_puts(tw, buf);
 #endif
@@ -179,7 +183,7 @@ static int display_size(void *handle, void *device, int width, int height,
     image_updatesize(img);
     return 0;
 }
-   
+
 /* flushpage */
 static int display_sync(void *handle, void *device)
 {
@@ -201,8 +205,8 @@ static int display_page(void *handle, void *device, int copies, int flush)
     IMAGE *img;
 #ifdef DISPLAY_DEBUG
     char buf[256];
-    sprintf(buf, "display_page(0x%x, 0x%x, copies=%d flush=%d)\n", 
-	handle, device, copies, flush);
+    sprintf(buf, "display_page(0x%x, 0x%x, copies=%d flush=%d)\n",
+        handle, device, copies, flush);
     text_puts(tw, buf);
 #endif
     img = image_find(handle, device);
@@ -212,7 +216,7 @@ static int display_page(void *handle, void *device, int copies, int flush)
 
 /* Poll the caller for cooperative multitasking. */
 /* If this function is NULL, polling is not needed */
-static int display_update(void *handle, void *device, 
+static int display_update(void *handle, void *device,
     int x, int y, int w, int h)
 {
     IMAGE *img;
@@ -221,15 +225,15 @@ static int display_update(void *handle, void *device,
     return poll();
 }
 
-int display_separation(void *handle, void *device, 
+int display_separation(void *handle, void *device,
    int comp_num, const char *name,
    unsigned short c, unsigned short m,
    unsigned short y, unsigned short k)
 {
     IMAGE *img;
 #ifdef DISPLAY_DEBUG
-    fprintf(stdout, "display_separation(0x%x, 0x%x, %d '%s' %d,%d,%d,%d)\n", 
-	handle, device, comp_num, name, (int)c, (int)m, (int)y, (int)k);
+    fprintf(stdout, "display_separation(0x%x, 0x%x, %d '%s' %d,%d,%d,%d)\n",
+        handle, device, comp_num, name, (int)c, (int)m, (int)y, (int)k);
 #endif
     img = image_find(handle, device);
     if (img)
@@ -237,7 +241,7 @@ int display_separation(void *handle, void *device,
     return 0;
 }
 
-display_callback display = { 
+display_callback display = {
     sizeof(display_callback),
     DISPLAY_VERSION_MAJOR,
     DISPLAY_VERSION_MINOR,
@@ -254,7 +258,6 @@ display_callback display = {
     display_separation
 };
 
-
 /*********************************************************************/
 
 /* program really starts at WinMain */
@@ -271,15 +274,15 @@ int new_main(int argc, char *argv[])
 
     memset(buf, 0, sizeof(buf));
     if (load_dll(&gsdll, buf, sizeof(buf))) {
-	text_puts(tw, "Can't load Ghostscript DLL\n");
-	text_puts(tw, buf);
-	text_puts(tw, "\n");
-	return 1;
+        text_puts(tw, "Can't load Ghostscript DLL\n");
+        text_puts(tw, buf);
+        text_puts(tw, "\n");
+        return 1;
     }
 
     if (gsdll.new_instance(&instance, NULL) < 0) {
-	text_puts(tw, "Can't create Ghostscript instance\n");
-	return 1;
+        text_puts(tw, "Can't create Ghostscript instance\n");
+        return 1;
     }
 
 #ifdef DEBUG
@@ -292,28 +295,28 @@ int new_main(int argc, char *argv[])
     gsdll.set_display_callback(instance, &display);
 
     /* insert display device defaults as first arguments */
-    {   int format = DISPLAY_COLORS_NATIVE | DISPLAY_ALPHA_NONE | 
-		DISPLAY_DEPTH_1 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
-	HDC hdc = GetDC(NULL);	/* get hdc for desktop */
-	int depth = GetDeviceCaps(hdc, PLANES) * GetDeviceCaps(hdc, BITSPIXEL);
-	sprintf(ddpi, "-dDisplayResolution=%d", GetDeviceCaps(hdc, LOGPIXELSY));
+    {   int format = DISPLAY_COLORS_NATIVE | DISPLAY_ALPHA_NONE |
+                DISPLAY_DEPTH_1 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
+        HDC hdc = GetDC(NULL);	/* get hdc for desktop */
+        int depth = GetDeviceCaps(hdc, PLANES) * GetDeviceCaps(hdc, BITSPIXEL);
+        sprintf(ddpi, "-dDisplayResolution=%d", GetDeviceCaps(hdc, LOGPIXELSY));
         ReleaseDC(NULL, hdc);
-	if (depth == 32)
- 	    format = DISPLAY_COLORS_RGB | DISPLAY_UNUSED_LAST | 
-		DISPLAY_DEPTH_8 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
-	else if (depth == 16)
- 	    format = DISPLAY_COLORS_NATIVE | DISPLAY_ALPHA_NONE | 
-		DISPLAY_DEPTH_16 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST |
-		DISPLAY_NATIVE_555;
-	else if (depth > 8)
- 	    format = DISPLAY_COLORS_RGB | DISPLAY_ALPHA_NONE | 
-		DISPLAY_DEPTH_8 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
-	else if (depth >= 8)
- 	    format = DISPLAY_COLORS_NATIVE | DISPLAY_ALPHA_NONE | 
-		DISPLAY_DEPTH_8 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
-	else if (depth >= 4)
- 	    format = DISPLAY_COLORS_NATIVE | DISPLAY_ALPHA_NONE | 
-		DISPLAY_DEPTH_4 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
+        if (depth == 32)
+            format = DISPLAY_COLORS_RGB | DISPLAY_UNUSED_LAST |
+                DISPLAY_DEPTH_8 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
+        else if (depth == 16)
+            format = DISPLAY_COLORS_NATIVE | DISPLAY_ALPHA_NONE |
+                DISPLAY_DEPTH_16 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST |
+                DISPLAY_NATIVE_555;
+        else if (depth > 8)
+            format = DISPLAY_COLORS_RGB | DISPLAY_ALPHA_NONE |
+                DISPLAY_DEPTH_8 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
+        else if (depth >= 8)
+            format = DISPLAY_COLORS_NATIVE | DISPLAY_ALPHA_NONE |
+                DISPLAY_DEPTH_8 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
+        else if (depth >= 4)
+            format = DISPLAY_COLORS_NATIVE | DISPLAY_ALPHA_NONE |
+                DISPLAY_DEPTH_4 | DISPLAY_LITTLEENDIAN | DISPLAY_BOTTOMFIRST;
         sprintf(dformat, "-dDisplayFormat=%d", format);
     }
     nargc = argc + 2;
@@ -328,10 +331,10 @@ int new_main(int argc, char *argv[])
 #endif
     code = gsdll.init_with_args(instance, nargc, nargv);
     if (code == 0)
-	code = gsdll.run_string(instance, start_string, 0, &exit_code);
+        code = gsdll.run_string(instance, start_string, 0, &exit_code);
     code1 = gsdll.exit(instance);
     if (code == 0 || (code == e_Quit && code1 != 0))
-	code = code1;
+        code = code1;
 #if defined(_MSC_VER) || defined(__BORLANDC__)
     } __except(exception_code() == EXCEPTION_STACK_OVERFLOW) {
         code = e_Fatal;
@@ -351,22 +354,20 @@ int new_main(int argc, char *argv[])
 
     exit_status = 0;
     switch (code) {
-	case 0:
-	case e_Quit:
-	    break;
-	case e_Fatal:
-	    exit_status = 1;
-	    break;
-	case e_Info:
-	default:
-	    exit_status = 255;
+        case 0:
+        case e_Quit:
+            break;
+        case e_Fatal:
+            exit_status = 1;
+            break;
+        case e_Info:
+        default:
+            exit_status = 255;
     }
-    
+
     return exit_status;
 }
 
-
-
 void
 set_font(void)
 {
@@ -379,7 +380,7 @@ set_font(void)
     fontsize = GetPrivateProfileInt(szIniSection, "FontSize", 10, szIniName);
 
     /* set font */
-    text_font(tw, fontname, fontsize); 
+    text_font(tw, fontname, fontsize);
 
     /* write ini file */
     WritePrivateProfileString(szIniSection, "FontName", fontname, szIniName);
@@ -387,7 +388,7 @@ set_font(void)
     WritePrivateProfileString(szIniSection, "FontSize", buf, szIniName);
 }
 
-int PASCAL 
+int PASCAL
 WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow)
 {
     int dll_exit_status;
@@ -398,20 +399,23 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmd
     int argc;
     LPSTR argv[MAXCMDTOKENS];
     LPSTR p;
+#ifndef WINDOWS_NO_UNICODE
+    LPSTR pstart;
+#endif
     char command[256];
     char *args;
     char *d, *e;
     char winposbuf[256];
     int len = sizeof(winposbuf);
     int x, y, cx, cy;
- 
+
     /* copy the hInstance into a variable so it can be used */
     ghInstance = hInstance;
 
     if (hPrevInstance) {
-	MessageBox((HWND)NULL,"Can't run twice", szAppName, 
-		MB_ICONHAND | MB_OK);
-	return FALSE;
+        MessageBox((HWND)NULL,"Can't run twice", szAppName,
+                MB_ICONHAND | MB_OK);
+        return FALSE;
     }
 
     /* If called with "gswin32c.exe arg1 arg2"
@@ -419,105 +423,117 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmd
      *    "arg1 arg2" if called with CreateProcess(NULL, command, ...)
      *    "arg2"      if called with CreateProcess(command, args, ...)
      * GetCommandLine() returns
-     *    ""gswin32c.exe" arg1 arg2" 
+     *    ""gswin32c.exe" arg1 arg2"
      *          if called with CreateProcess(NULL, command, ...)
-     *    "  arg1 arg2"      
+     *    "  arg1 arg2"
      *          if called with CreateProcess(command, args, ...)
-     * Consequently we must use GetCommandLine() 
+     * Consequently we must use GetCommandLine()
      */
+#ifdef WINDOWS_NO_UNICODE
     p = GetCommandLine();
+#else
+    {
+        wchar_t *uni = GetCommandLineW();
+        pstart = p = malloc(wchar_to_utf8(NULL, uni));
+        if (p != NULL)
+            wchar_to_utf8(p, uni);
+    }
+#endif
 
     argc = 0;
     args = (char *)malloc(lstrlen(p)+1);
     if (args == (char *)NULL) {
-	fprintf(stdout, "Insufficient memory in WinMain()\n");
-	return 1;
+        fprintf(stdout, "Insufficient memory in WinMain()\n");
+        return 1;
     }
-   
+
     /* Parse command line handling quotes. */
     d = args;
     while (*p) {
-	/* for each argument */
+        /* for each argument */
 
-	if (argc >= MAXCMDTOKENS - 1)
-	    break;
+        if (argc >= MAXCMDTOKENS - 1)
+            break;
 
         e = d;
         while ((*p) && (*p != ' ')) {
-	    if (*p == '\042') {
-		/* Remove quotes, skipping over embedded spaces. */
-		/* Doesn't handle embedded quotes. */
-		p++;
-		while ((*p) && (*p != '\042'))
-		    *d++ =*p++;
-	    }
-	    else 
-		*d++ = *p;
-	    if (*p)
-		p++;
+            if (*p == '\042') {
+                /* Remove quotes, skipping over embedded spaces. */
+                /* Doesn't handle embedded quotes. */
+                p++;
+                while ((*p) && (*p != '\042'))
+                    *d++ =*p++;
+            }
+            else
+                *d++ = *p;
+            if (*p)
+                p++;
         }
-	*d++ = '\0';
-	argv[argc++] = e;
+        *d++ = '\0';
+        argv[argc++] = e;
 
-	while ((*p) && (*p == ' '))
-	    p++;	/* Skip over trailing spaces */
+        while ((*p) && (*p == ' '))
+            p++;	/* Skip over trailing spaces */
     }
     argv[argc] = NULL;
 
-    if (strlen(argv[0]) == 0) {
-	GetModuleFileName(hInstance, command, sizeof(command)-1);
-	argv[0] = command;
-    }
+#ifndef WINDOWS_NO_UNICODE
+    free(pstart);
+#endif
 
+    if (strlen(argv[0]) == 0) {
+        GetModuleFileName(hInstance, command, sizeof(command)-1);
+        argv[0] = command;
+    }
 
     tw = text_new();
     if (tw == NULL) {
-	MessageBox((HWND)NULL, "Can't create text window", 
-		szAppName, MB_OK | MB_ICONSTOP);
- 	return 1;
+        MessageBox((HWND)NULL, "Can't create text window",
+                szAppName, MB_OK | MB_ICONSTOP);
+        return 1;
     }
 
     /* start up the text window */
     if (!hPrevInstance) {
-	HICON hicon = LoadIcon(hInstance, (LPSTR)MAKEINTRESOURCE(GSTEXT_ICON));
-	text_register_class(tw, hicon);
+        HICON hicon = LoadIcon(hInstance, (LPSTR)MAKEINTRESOURCE(GSTEXT_ICON));
+        text_register_class(tw, hicon);
     }
     set_font();
     text_size(tw, 80, 80);
     text_drag(tw, "(", ") run\r");
     if (win_get_reg_value("Text", winposbuf, &len) == 0) {
-	if (sscanf(winposbuf, "%d %d %d %d", &x, &y, &cx, &cy) == 4)
-	    text_setpos(tw, x, y, cx, cy);
+        if (sscanf(winposbuf, "%d %d %d %d", &x, &y, &cx, &cy) == 4)
+            text_setpos(tw, x, y, cx, cy);
     }
 
     /* create the text window */
     if (text_create(tw, szAppName, cmdShow))
-	exit(1);
+        exit(1);
 
     hwndtext = text_get_handle(tw);
     image_textwindow(hwndtext);
 
     dll_exit_status = new_main(argc, argv);
-    
+
     if (dll_exit_status && !tw->quitnow) {
-	/* display error message in text window */
-	MSG msg;
-	text_puts(tw, "\nClose this window with the close button on the title bar or the system menu.\n");
-	if (IsIconic(text_get_handle(tw)))
-	    ShowWindow(text_get_handle(tw), SW_SHOWNORMAL);
-	BringWindowToTop(text_get_handle(tw));  /* make text window visible */
-	FlashWindow(text_get_handle(tw), TRUE);
-	/* Wait until error message is read */
-	while (!tw->quitnow && GetMessage(&msg, (HWND)NULL, 0, 0)) {
-	    TranslateMessage(&msg);
-	    DispatchMessage(&msg);
-	}
+        /* display error message in text window */
+        MSG msg;
+        text_puts(tw, "\nClose this window with the close button on the title bar or the system menu.\n");
+        if (IsIconic(text_get_handle(tw)))
+            ShowWindow(text_get_handle(tw), SW_SHOWNORMAL);
+        BringWindowToTop(text_get_handle(tw));  /* make text window visible */
+        FlashWindow(text_get_handle(tw), TRUE);
+        /* Wait until error message is read */
+        while (!tw->quitnow && GetMessage(&msg, (HWND)NULL, 0, 0)) {
+            TranslateMessage(&msg);
+            DispatchMessage(&msg);
+        }
     }
 
     /* Save the text window size */
     if (text_getpos(tw, &x, &y, &cx, &cy) == 0) {
-	sprintf(winposbuf, "%d %d %d %d", x, y, cx, cy);
-	win_set_reg_value("Text", winposbuf);
+        sprintf(winposbuf, "%d %d %d %d", x, y, cx, cy);
+        win_set_reg_value("Text", winposbuf);
     }
 
     text_destroy(tw);
@@ -525,6 +541,3 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmd
 
     return dll_exit_status;
 }
-
-
-
